@@ -1,5 +1,5 @@
 #
-#  Copyright (c) 1997-2000 The Protein Laboratory, University of Copenhagen
+#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
 #  All rights reserved.
 #
 #  Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
 #  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 #  SUCH DAMAGE.
 #
-#  $Id: buttons2.pl,v 1.8 2001/10/25 11:21:10 dk Exp $
+#  $Id: buttons2.pl,v 1.11 2003/08/08 11:43:57 dk Exp $
 #
 
 =pod 
@@ -41,16 +41,15 @@ pastes its image into the clipboard.
 
 use Prima qw(Buttons StdBitmap), Application => { name => 'Buttons sample' };
 
-my $w = Prima::Window-> create(
+my $w = Prima::MainWindow-> create(
   text=> "Handmade buttons",
   size => [ 300, 200],
-  onDestroy => sub { $::application-> close},
   centered => 1,
   popupItems => [['Hallo!' => '']],
 );
 
 $w-> insert( CheckBox =>
-   rect => [ 10, 150, 180, 180],
+   origin => [ 10, 150],
    selectable => 1,
    text => "~Check box",
    hint => 'Check box!',
@@ -58,7 +57,7 @@ $w-> insert( CheckBox =>
 );
 
 $w-> insert( Radio =>
-   rect => [ 190, 150, 300, 180],
+   origin => [ 190, 150],
    selectable => 1,
    text => "~Radio button",
    hint => 'Radio!',
@@ -82,6 +81,7 @@ $w-> insert( Button =>
    hint => 'Sly button',
 );
 
+
 $w-> insert( Button =>
    origin  => [ 10, 10],
    text => "Disab~led",
