@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: apc_font.c,v 1.88 2004/12/13 15:54:17 dk Exp $
+ * $Id: apc_font.c,v 1.89 2005/03/30 08:56:32 dk Exp $
  */
 
 /***********************************************************/
@@ -1529,6 +1529,12 @@ AGAIN:
    return true;
 }
 
+Bool
+prima_core_font_encoding( char * encoding)
+{
+   return hash_fetch( encodings, encoding, strlen( encoding)) != NULL;
+}
+
 Bool
 apc_font_pick( Handle self, PFont source, PFont dest)
 {
